CREATE TABLE Person(
	SVNR INTEGER PRIMARY KEY,
	PNR INTEGER UNIQUE NOT NULL,
	NNAME VARCHAR(30) NOT NULL,
	RAUM VARCHAR(20) NOT NULL
);

CREATE TABLE SWEntwickler(
	SVNR INTEGER PRIMARY KEY REFERENCES Person(SVNR)
);

CREATE TABLE Qualitaetsmanager(
	SVNR INTEGER PRIMARY KEY REFERENCES Person(SVNR)
);

CREATE TABLE Programmiersprache(
	NAME VARCHAR(30) PRIMARY KEY
);

CREATE TABLE beherrscht(
	SVNR INTEGER REFERENCES SWEntwickler(SVNR),
	NAME VARCHAR(30) REFERENCES Programmiersprache(NAME),
	LEVEL INTEGER NOT NULL,
	PRIMARY KEY (SVNR, NAME)
);

CREATE TABLE Software(
	ID INTEGER PRIMARY KEY,
	TITEL VARCHAR(20) NOT NULL,
	TYP VARCHAR(20) NOT NULL
);

CREATE TABLE Version(
	ID INTEGER REFERENCES Software(ID),
	VNR NUMERIC,
	PRIMARY KEY (ID, VNR)
);

CREATE TABLE Build(
	ID INTEGER,
	VNR NUMERIC,
	BNR NUMERIC,
	PRIMARY KEY (ID, VNR, BNR),
	FOREIGN KEY(ID, VNR) REFERENCES Version(ID, VNR)
);

CREATE TABLE Paket(
	NAME VARCHAR(30),
	VERSION NUMERIC,
	QUELLE VARCHAR(50) NOT NULL,
	PRIMARY KEY (NAME, VERSION)
);

CREATE TABLE benoetigt(
	NAME VARCHAR(30),
	VERSION NUMERIC,
	VORAUSSETZUNG_NAME VARCHAR(30),
	VORAUSSETZUNG_VERSION NUMERIC,
	PRIMARY KEY(NAME, VERSION, VORAUSSETZUNG_NAME, VORAUSSETZUNG_VERSION),
	FOREIGN KEY(NAME, VERSION) REFERENCES Paket(NAME, VERSION),
	FOREIGN KEY(VORAUSSETZUNG_NAME, VORAUSSETZUNG_VERSION) REFERENCES Paket(NAME, VERSION)
);

CREATE TABLE beinhaltet(
	ID INTEGER,
	VNR NUMERIC,
	PAKETNAME VARCHAR(30) NOT NULL,
	PAKETVERSION NUMERIC NOT NULL,
	PRIMARY KEY (ID,VNR,PAKETNAME,PAKETVERSION),
	FOREIGN KEY (ID,VNR) REFERENCES Version(ID,VNR),
	FOREIGN KEY (PAKETNAME,PAKETVERSION) REFERENCES Paket(NAME,VERSION)
);

CREATE TABLE File(
	NAME VARCHAR(30), 
	VERSION NUMERIC,
	FNAME VARCHAR(30),
	INHALT TEXT NOT NULL,
	FTYPE VARCHAR(4) NOT NULL,
	PRIMARY KEY (NAME, VERSION, FNAME),
	FOREIGN KEY (NAME, VERSION) REFERENCES Paket(NAME, VERSION)
);

CREATE TABLE Sourcecode(
	NAME VARCHAR(30),
	VERSION NUMERIC,
	FNAME VARCHAR(30),
	LOC INTEGER NOT NULL,
	PROG_NAME VARCHAR(30) REFERENCES Programmiersprache(NAME) NOT NULL,
	PRIMARY KEY (NAME, VERSION, FNAME),
	FOREIGN KEY (NAME, VERSION, FNAME) REFERENCES File(NAME, VERSION, FNAME)
);

CREATE TABLE BugReport(
	TRNR INTEGER PRIMARY KEY,
	EINGANG TIMESTAMP NOT NULL, 
	BESCHREIBUNG TEXT NOT NULL,
	ID INTEGER  NOT NULL,
	VNR NUMERIC NOT NULL,
	BNR NUMERIC NOT NULL,
	FOREIGN KEY (ID, VNR, BNR) REFERENCES Build(ID, VNR, BNR)
);

CREATE TABLE Bug(
	TRNR INTEGER PRIMARY KEY REFERENCES BugReport(TRNR),
	SVNR INTEGER REFERENCES Qualitaetsmanager(SVNR) NOT NULL,
	DATUM TIMESTAMP NOT NULL
);

CREATE TABLE BehobenerBug(
	TRNR INTEGER PRIMARY KEY REFERENCES Bug(TRNR),
	KOMMENTAR TEXT NOT NULL,
	SVNR INTEGER REFERENCES SWEntwickler(SVNR) NOT NULL,
	DATUM TIMESTAMP NOT NULL
);

CREATE TABLE geprueft(
	TRNR INTEGER PRIMARY KEY REFERENCES BugReport(TRNR),
	SVNR INTEGER REFERENCES Qualitaetsmanager(SVNR) NOT NULL,
	DATUM TIMESTAMP NOT NULL,
	BEWERTUNG VARCHAR(40) NOT NULL
);